<?php

/**
 * @file
 * Definition of Drupal\cryptolog\CryptologSubscriber.
 */

namespace Drupal\cryptolog;

use Drupal\Component\Utility\Crypt;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Cryptolog subscriber for controller requests.
 */
class CryptologSubscriber implements EventSubscriberInterface {

  /**
   * The cache identifier.
   */
  const KEY = 'cryptolog';

  /**
   * The cache time to live in seconds.
   */
  const TTL = 86400;

  /**
   * The cache backend; currently used only for Memcache.
   */
  protected $backend;

  /**
   * The cached salt and day of year.
   */
  protected $cache;

  /**
   * Construct the cryptolog subscriber.
   */
  public function __construct($memcache = NULL) {
    if ($memcache) {
      $this->backend = $memcache->get(self::KEY);
    }
    $this->cache = $this->getCache();
    $day = gmdate('zY');
    if (!$this->cache || $this->cache['day'] !== $day) {
      $this->cache = array('day' => $day, 'salt' => Crypt::randomBytes(32));
      $this->setCache($this->cache);
    }
  }

  /**
   * Fetches salt and day of year from the memory-backed cache.
   */
  public function getCache() {
    if ($this->backend) {
      $cache = $this->backend->get(self::KEY);
      return $cache ? $cache->data : FALSE;
    }
    if (extension_loaded('apcu')) {
      return apcu_fetch(self::KEY);
    }
    if (extension_loaded('xcache')) {
      return xcache_get(self::KEY);
    }
    $cache = \Drupal::cache()->get(self::KEY);
    return $cache ? $cache->data : FALSE;
  }

  /**
   * Registers the methods in this class that should be listeners.
   *
   * @return array
   *   An array of event listener definitions.
   */
  static function getSubscribedEvents() {
    $events[KernelEvents::REQUEST][] = array('setClientIP', 222);
    return $events;
  }

  /**
   * Stores salt and day of year in the memory-backed cache.
   */
  public function setCache($value) {
    if ($this->backend) {
      return $this->backend->set(self::KEY, $value, self::TTL + REQUEST_TIME);
    }
    if (extension_loaded('apcu')) {
      return apcu_store(self::KEY, $value, self::TTL);
    }
    if (extension_loaded('xcache')) {
      return xcache_set(self::KEY, $value, self::TTL);
    }
    return \Drupal::cache()->set(self::KEY, $value, self::TTL + REQUEST_TIME);
  }

  /**
   * Sets the client IP address.
   */
  public function setClientIP(GetResponseEvent $event) {
    $request = $event->getRequest();
    // Generate a keyed hash in IPv6 address notation.
    $request->server->set('REMOTE_ADDR', inet_ntop(hash_hmac('md5', $request->getClientIP(), $this->cache['salt'], TRUE)));
  }

}
